{$A+,B-,E-,F-,I-,N-,O-,R-,S-,V-,X- }

{$IFDEF P286 } {$G+ } {$ELSE } {$G- } {$ENDIF }

{$IFNDEF DEBUG } {$D-,L-,R-,S- } {$ENDIF }


{**************************************************************************
 *                                                                        *
 *                         X S E R   v 1.56     date: 14/07/93            *
 *                                                                        *
 *                          Par T.JAOUEN                                  *
 *                                                                        *
 *                                                                        *
 *                   Gestion des COM1  l'infini,                         *
 *                  avec ou sans IRQ (partageable).                       *
 *                                                                        *
 *                    Ecrit en Turbo Pascal 6.0                           *
 *                                                                        *
 *                                  (C) Copyright 1991-93, Thierry JAOUEN *
 *                                                                        *
 *                                                                        *
 *                          Thierry JAOUEN                                *
 *                     2, avenue Winston Churchill                        *
 *                       94220 Charenton le pont  (FRANCE)                *
 *                                                                        *
 **************************************************************************}


Unit XSer;

Interface



{--------------------------------------------------------------------------}
{  Les procedures qui suivent ne servent qu' gerer les ports sries sans  }
{  IRQ.                                                                    }
{--------------------------------------------------------------------------}

Procedure CheckUp;       { Gre les ports sans IRQ                         }

Procedure RanCheckUp;    { Idem, mais les registres sont sauvegards       }

Procedure IntCheckUp;    { Idem, mais avec un "IRET"  la fin              }

Function  InstCheckUp( Speed : Byte ) : Boolean;

   { Installe un "CheckUp" sur l'interruption 08h (IRQ 0).                 }
   { La vitesse est: 18.20648193*2^Speed interruptions par seconde, avec   }
   { Speed compris entre 0 et 15.                                          }

Procedure KillCheckUp;   { Dsinstalle le "CheckUp"                        }

Function  InstCheckUpAT( Speed : Byte ) : Boolean;

   { Installe un "CheckUp" sur l'interruption 70h (IRQ 8) d'un AT.         }
   { La vitesse est: 2^(Speed+1) interruptions par seconde, avec Speed     }
   { compris entre 0 et 15.                                                }

Procedure KillCheckUpAT; { Dsinstalle le "CheckUpAT"                      }


Procedure KeepXSer;      { deconnecte le noyau de communication }

Procedure ResumeXSer;    { retablit le noyau de communication (aprs KeepXSer) }



{--------------------------------------------------------------------------}
{                                                                          }
{                          Constantes & Types                              }
{                                                                          }
{--------------------------------------------------------------------------}


Const { Exemples de vitesses possibles -----------------------------------}

      BAUD75     = 115200 DIV 75;
      BAUD110    = 115200 DIV 110;    { <-- le rsultat n'est pas entier! }
      BAUD150    = 115200 DIV 150;
      BAUD300    = 115200 DIV 300;
      BAUD600    = 115200 DIV 600;
      BAUD1200   = 115200 DIV 1200;
      BAUD2400   = 115200 DIV 2400;
      BAUD4800   = 115200 DIV 4800;
      BAUD9600   = 115200 DIV 9600;
      BAUD19200  = 115200 DIV 19200;
      BAUD38400  = 115200 DIV 38400;

      { Voir "ComX.Init" et "ComX.SetBaud" }


      { Parametres de communication --------------------------------------}

      NOPARITY   = $00;
      ODDPARITY  = $08;
      EVENPARITY = $18;

      DATA5BITS  = $00;
      DATA6BITS  = $01;
      DATA7BITS  = $02;
      DATA8BITS  = $03;

      STOP1BIT   = $00;
      STOP2BITS  = $04;

      { Voir "ComX.Init" et "ComX.SetParam" }


      { Differents tats du modem ----------------------------------------}

      { "DL" pour "DELTA" => changement d'tat du modem }

      DL_RTS     = $01;        { Ready To Send }
      DL_CTS     = DL_RTS;     { Clear To Send }
      DL_DSR     = $02;        { Data Set Ready }
      DL_RI      = $04;        { Ring Indicator }
      DL_DCD     = $08;        { Data Carrier Detect }

      { "MD" pour "MODEM" }

      MD_RTS     = $10;        { Ready To Send }
      MD_CTS     = MD_RTS;     { Clear To Send }
      MD_DSR     = $20;        { Data Set Ready }
      MD_RI      = $40;        { Ring Indicator }
      MD_DCD     = $80;        { Data Carrier Detect }

      { Voir "ComX.NewModemState" et "ComX.GetModemState" }


      { Diffrents etats de la ligne -------------------------------------}

      { "LN" pour "LINE" }

      LN_RDA     = $01;   { Un caractere est diponible                 (1)}
      LN_OVRN    = $02;   { (Erreur) Engorgement du tampon entre      (2)}
      LN_PAR_E   = $04;   { (Erreur) Parit                               }
      LN_FRA_E   = $08;   { (Erreur) encadrement (bit "STOP")             }
      LN_BREAK   = $10;   { (Erreur) Dtection de "BREAK"                 }
      LN_THRE    = $20;   { Registre d'attente de transmission vide    (1)}
      LN_TSRE    = $40;   { Registre de decalage de transimission vide (1)}
                          { Bit 7 toujours 0                              }
      LN_ERROR   = LN_OVRN+LN_PAR_E+LN_FRA_E+LN_BREAK; { Conditions d'erreurs }

      { (1) => L'tat importe peu pour vous, il ne concerne que XSER.     }
      { (2) => Fix par XSER lorsque le tampon entre n'a pus recevoir    }
      {        une donne car il tait plein.                             }

      { Voir "ComX.LineError" et "ComX.GetLineState" }


      { Contrle du modem ------------------------------------------------}

      { "CT" pour "CTRL" }

      CT_DTR     = $01;        { Data Transmit Ready }
      CT_RTS     = $02;        { Ready To Send }
      CT_CTS     = CT_RTS;     { Bis: Clear To Send }
      CT_OUT1    = $04;        { ??? }
      CT_OUT2    = $08;        { ??? et valide IT }

      { Les bits 4  7, ne sont pas manipulable ici }

      { Voir "ComX.GetModemCtrl" et "ComX.SetModemCtrl" }


{--------------------------------------------------------------------------}
{                                                                          }
{                      L ' o b j e t    C o m X                            }
{                                       -------                            }
{                                                                          }
{--------------------------------------------------------------------------}

Type BufferRec = Record
         Addr    : Pointer; { Adresse de la mmoire tampon alloue         }
         SegBuf  : Word;    { Position du tampon (Segment)                 }
         Size    : Word;    { Longueur du Tampon (1 minimun)               }
         Read    : Word;    { OffSet en lecture dans le tampon             }
         Write   : Word;    { OffSet en ecriture dans le tampon            }
         Nb      : Word;    { Nombre de donnes  lire                     }
       End;


     PComX = ^ComX;

     ComX = Object

             { Vous ne devrez pas avoir besoin d'acceder directement }
             {  ces donnes. Les mthodes font cela trs bien.      }

         RegBase    : Word;       { Adresse registre UART                  }
         nIRQ       : Byte;       { nIRQ associ                          }
         Unused     : Byte;       { inutilis                              }
         Input      : BufferRec;  { Les entres                            }
         Output     : BufferRec;  { Les sorties                            }
         Chain      : Boolean;    { Chainage des sorties                   }
         DataLost   : Boolean;    { Donne perdu car entres pleines       }
         LineSt     : Byte;       { Etat de la ligne si NLineSt=TRUE       }
         NLineSt    : Boolean;    { changement d'etat de la ligne          }
         ModemSt    : Byte;       { Etat du modem si NModemSt=TRUE         }
         NModemSt   : Boolean;    { changement d'etat du modem             }
         KpMdCtrl   : Byte;       { Sauvegarde lors de "KeepXSer"          }
         KpITValid  : Byte;       { Sauvegarde lors de "KeepXSer"          }
         Prec       : PComX;      { Objet prcdent (sur la mme IRQ)      }
         Next       : PComX;      { Objet suivant (sur la mme IRQ)        }


  Function  Init( RegUART    : Word;
                  NumIRQ     : Byte;
                  Baud       : Word;
                  Parameters : Byte;
                  SizeIn     : Word;
                  SizeOut    : Word  ) : Boolean;
  Procedure Done;

  Procedure SetBaud( Baud : Word );              { changer de vitesse         }
  Procedure SetParam( Parameters : Byte );       { changer les paramtres     }
  Procedure SetBreak( BrkOn : Boolean );         { Break ON/OFF               }
  Procedure SetLoop( LoopOn : Boolean );         { Loop ON/OFF                }

  Procedure SetModemCtrl( Parameters : Byte );   { contrle du modem          }
  Function  GetModemCtrl : Byte;                 { l'etat du controleur       }

  Function  LineError : Boolean;                 { TRUE=>erreur sur la ligne  }
  Function  GetLineState : Byte;                 { renvoi l'etat de la ligne  }

  Function  NewModemState : Boolean;             { TRUE=>nouvelle etat        }
  Function  GetModemState : Byte;                { renvoi l'etat du modem     }

  Function  ReceiveCount : Word;                 { donnes dans tampon entre }
  Function  ReceiveAvail : Word;                 { place disponible           }
  Function  ReceiveChar( Var C : Char ) : Boolean; { lire un caractere        }
  Function  Receive( Var p; LgMax : Word ) : Word; { lire un block            }
  Procedure ReceiveClear;                        { effacer le tampon entre   }

  Function  TransmitCount : Word;                { donnes dans tampon sortie }
  Function  TransmitAvail : Word;                { place disponible           }
  Function  TransmitChar( C : Char ) : Boolean;  { envoi un caractre         }
  Function  Transmit( Var p; Lg : Word ) : Word; { envoyer un block           }
  Procedure TransmitClear;                       { effacer le tampon sortie   }

       End;




{--------------------------------------------------------------------------}
{                                                                          }
{  Un mcanisme de sortie a t prvu pour supprimer et rtablir tous ce   }
{  qui devraient l'tre: Il ne sert que si des ports n'ont pas t         }
{  ferms, ou si le "CheckUp" n'a pas t dsinstall.                     }
{                                                                          }
{--------------------------------------------------------------------------}


Implementation


Const MAXIRQ = 16;  { Nombre d'Irq maximun (Valeur inchangeable) }

      INPUTSIZEMAX  = 64512;  { Par dfaut }
      OUTPUTSIZEMAX = 64512;  { ... }

      INOUTPUT   = 0;    { Reception/Transmission des donnes (DLAB=0) }
      ITVALID    = 1;    { Autorisation des interruptions     (DLAB=0) }
      ITIDEN     = 2;    { Identification des interruptions }
      LINECTRL   = 3;    { Contrle de la ligne }
      MODEMCTRL  = 4;    { Contrle des signaux modem }
      LINESTATE  = 5;    { Etat de la ligne }
      MODEMSTATE = 6;    { Etat des signaux modem }
      UNUSED     = 7;    { Inutilis }

      LINEBAUD   = 0;    { Vitesse communication (WORD)       (DLAB=1) }

      { ITVALID  : }

      ITINPUT    = $01;  { IT en reception }
      ITOUTPUT   = $02;  { IT en sortie }
      ITERROR    = $04;  { IT si erreur }
      ITMODEM    = $08;  { IT si changement d'etat du modem }

      { LINECTRL : }

      BREAKON    = $40;  { Gnrateur de BREAK }
      DLAB       = $80;  { DLAB ... }

      { ITIDEN }

      IDMODEM    = $00;
      IDOUTPUT   = $02;
      IDINPUT    = $04;
      IDERROR    = $06;

      { MODEMCTRL }

      CTRL_LOOP  = $10;  { Boucle UART sur lui-mme (pour tester) }


      EOI        = $20;  { End Of Interrupt }


Type IrqDef = Record
         IntVec   : Pointer;   { Vecteur interruption sauvegard }
         IrqMsk   : Boolean;   { TRUE=>L'interruption tait masqu }
         UnUse    : Array[1..3] Of Byte;  { inutilis }
         FirstCom : PComX;     { Premiere Com sur cet Irq }
         LastCom  : PComX;     { Derniere ... }
       End;
     PIrqDef = ^IrqDef;

     { Note: SizeOf(IrqDef)=16 doit tre constant. Cela accelere le }
     {       calcul de l'adresse des donnes. Dans une procedure en }
     {       assembleur, on considere toujours SizeOf(IrqDef)=16    }
     {       sans verifier ! : Ne changez pas la taille au hasard ! }



Var SaveExit : Pointer;     { sauvegarde adresse ExitProc }
    TabIrq   : Array[0..MAXIRQ-1] Of IrqDef;


{ Procedure gerant les ports sans irq                                   }
{ Attention: Ne sauvegarde pas les registres, et ne restore pas DS      }
{            Utiliser "RanCheckUp" pour cela                            }
{            Pour les interruptions: "IntCheckUp"                       }
{     Call "IntCheckUp" -?-> Call "RanCheckUp" -> Call "CheckUp"        }

Var CheckUpImbr : Word;

Procedure CheckUp; Assembler;
Asm
    DEC  [CheckUpImbr]             { Signale pour entrer }
    JZ   @LIBRE
    JMP  @OCCUPE

@LIBRE:

    PUSH DS                        { Sauve }

    MOV  SI, OFFSET TabIrq

    LDS  SI, [SI+IrqDef.FirstCom]  { DS:SI = Adresse FirstCom }

    JMP  @NEXTCOM0


@BOUCLE:

    { Regarde l'etat du modem }

    MOV  DX, [SI+ComX.RegBase]
    ADD  DX, MODEMSTATE
    IN   AL, DX                    { Lire l'etat du modem }
    TEST AL, DL_RTS+DL_DSR+DL_RI+DL_DCD
    JZ   @01                       { Saut si rien de neuf }
    MOV  [SI+ComX.ModemSt], AL     { Garde l'etat }
    MOV  [SI+ComX.NModemSt], TRUE  { Signale la nouveaut }
@01:
    { Regarde l'etat de la ligne }

    MOV  DX, [SI+ComX.RegBase]
    ADD  DX, LINESTATE
    IN   AL, DX                    { Lire l'etat de la ligne }
    TEST AL, LN_ERROR
    JZ   @02                       { Saute si pas d'erreur }
    MOV  [SI+ComX.LineSt], AL      { Garde }
    MOV  [SI+ComX.NLineSt], TRUE   { Signale }
@02:
    { Quelque chose  lire ? }
    TEST AL, LN_RDA
    JZ   @03B                      { Saute si rien  lire }
    PUSH AX                        { Sauve }
    MOV  DX, [SI+ComX.RegBase]
    IN   AL, DX                    { Prendre la donne }
    MOV  BX, [SI+ComX.Input.Nb]
    CMP  BX, [SI+ComX.Input.Size]
    JAE  @IN02                     { Pas de place dans le buffer !!! }
    MOV  ES, [SI+ComX.Input.SegBuf]
    MOV  BX, [SI+ComX.Input.Write]
    MOV  [ES:BX], AL               { Ecrire la donne dans le buffer }
    INC  BX
    CMP  BX, [SI+ComX.Input.Size]
    JB   @IN01
    XOR  BX, BX
@IN01:
    MOV  [SI+ComX.Input.Write], BX
    INC  [SI+ComX.Input.Nb]
    JMP  @03
@IN02:                             { Plus de place dans le buffer }
    MOV  [SI+ComX.DataLost], TRUE  { Signale la perte }
@03:
    POP  AX                        { recuperer l'etat sauv }
@03B:
    { Prt  emettre ? }
    TEST AL, LN_THRE
    JZ   @04                       { Sortie pas libre }
    CMP  [SI+ComX.Chain], FALSE
    JE   @04                       { Saut si pas de chainage: attendre }
    CMP  [SI+ComX.Output.Nb], 0
    JE   @OUT02                    { Saut si buffer vide }
    MOV  ES, [SI+ComX.Output.SegBuf]
    MOV  BX, [SI+ComX.Output.Read]
    MOV  AL, [ES:BX]               { Saisie la donne du buffer }
    MOV  DX, [SI+ComX.RegBase]
    OUT  DX, AL                    { Envoi la donne }
    INC  BX
    CMP  BX, [SI+ComX.Output.Size]
    JB   @OUT01
    XOR  BX, BX
@OUT01:
    MOV  [SI+ComX.Output.Read], BX
    DEC  [SI+ComX.Output.Nb]       { -1 donne dans le buffer }
    JMP  @04
@OUT02:                            { Buffer vide }
    MOV  [SI+ComX.Chain], FALSE    { Chainage hors fonction }
@04:

    LDS  SI, [SI+ComX.Next]        { DS:SI = ComX.Next }
@NEXTCOM0:
    MOV  AX, DS
    OR   AX, SI
    JZ   @FINBOUCLE                { Saut si DS:SI=Nil }
    JMP  @BOUCLE                   { DS:SI = Adresse ComX (Objet) }

@FINBOUCLE:

    POP  DS                        { Restore }

@OCCUPE:

    INC  [CheckUpImbr]             { Signale la sortie }

End;



{ Procedure pour des acces aleatoires au CheckUp }
{ C'est  dire: Aucun registre n'est modifi     }

Procedure RanCheckUp; Assembler;
Asm
  PUSHF
  PUSH AX
  PUSH BX
  PUSH DX
  PUSH SI
  PUSH DS
  PUSH ES

  MOV  AX, SEG @DATA
  MOV  DS, AX                      { Restore }
  CALL CheckUp

  POP  ES
  POP  DS
  POP  SI
  POP  DX
  POP  BX
  POP  AX
  POPF
End;


{ Procedure pour des acces aleatoires via une interruption }

Procedure IntCheckUp; Assembler;
Asm
  PUSH AX
  PUSH BX
  PUSH DX
  PUSH SI
  PUSH DS
  PUSH ES

  MOV  AX, SEG @DATA
  MOV  DS, AX                      { Restore }
  CALL CheckUp

  POP  ES
  POP  DS
  POP  SI
  POP  DX
  POP  BX
  POP  AX

  IRET
End;



{ Gestionnaire de toutes les interruptions series }
{ En entre AL=Offset dans TabIrq }
{ Cette procedure n'est pas une procedure.... }

Procedure IntSerialKernel; Far; Assembler;
Asm
                                   { PUSH AX   a eu lieu avant le saut... }
    PUSH BX
    PUSH CX
    PUSH DX
    PUSH SI
    PUSH DS
    PUSH ES

    { le numro de l'IRQ (Offset) en cours est dans AL ... }

    XOR  AH, AH

    MOV  CH, TRUE                  { Indique "aucune int com trait" }

@REPART:
    PUSH AX                        { Sauve (on verra  la fin) }

    MOV  SI, AX

    MOV  AX, SEG TabIrq
    MOV  DS, AX

    ADD  SI, OFFSET TabIrq         { DS:SI Pointe sur IrqDef courant }

    LDS  SI, [SI+IrqDef.FirstCom]  { DS:SI = Adresse FirstCom }

    MOV  CL, TRUE                  { Indique "pas d'int com trait" }

    JMP  @NEXTCOM0

@BOUCLE:
    XOR  CL, CL                    { Indique au moins une int com trait }
    CMP  AL, IDINPUT
    JE   @INPUT
    CMP  AL, IDOUTPUT
    JE   @OUTPUT
    CMP  AL, IDERROR
    JE   @ERROR
    CMP  AL, IDMODEM
    JE   @MODEM
    JMP  @NEXT                     { Interruption inconnu ! }
                                  
@INPUT:                            { Donne recu }
    MOV  DX, [SI+ComX.RegBase]
    IN   AL, DX                    { Prendre la donne }
    MOV  BX, [SI+ComX.Input.Nb]
    CMP  BX, [SI+ComX.Input.Size]
    JAE  @IN02                     { Pas de place dans le buffer !!! }
    MOV  ES, [SI+ComX.Input.SegBuf]
    MOV  BX, [SI+ComX.Input.Write]
    MOV  [ES:BX], AL               { Ecrire la donne dans le buffer }
    INC  BX
    CMP  BX, [SI+ComX.Input.Size]
    JB   @IN01
    XOR  BX, BX
@IN01:
    MOV  [SI+ComX.Input.Write], BX
    INC  [SI+ComX.Input.Nb]
    JMP  @NEXT
@IN02:                             { Plus de place dans le buffer }
    MOV  [SI+ComX.DataLost], TRUE  { Signale la perte }
    JMP  @NEXT

@OUTPUT:                           { Sortie libre }
    CMP  [SI+ComX.Output.Nb], 0
    JE   @OUT02                    { Saut si buffer vide }
    MOV  ES, [SI+ComX.Output.SegBuf]
    MOV  BX, [SI+ComX.Output.Read]
    MOV  AL, [ES:BX]               { Saisie la donne du buffer }
    MOV  DX, [SI+ComX.RegBase]
    OUT  DX, AL                    { Envoi la donne }
    INC  BX
    CMP  BX, [SI+ComX.Output.Size]
    JB   @OUT01
    XOR  BX, BX
@OUT01:
    MOV  [SI+ComX.Output.Read], BX
    DEC  [SI+ComX.Output.Nb]       { -1 donne dans le buffer }
    JMP  @NEXT
@OUT02:                            { Buffer vide }
    MOV  [SI+ComX.Chain], FALSE    { Chainage hors fonction }
    JMP  @NEXT

@ERROR:                            { Erreur sur la ligne }
    MOV  DX, [SI+ComX.RegBase]
    ADD  DX, LINESTATE
    IN   AL, DX
    MOV  [SI+ComX.LineSt], AL
    MOV  [SI+ComX.NLineSt], TRUE   { Etat d'erreur sur la ligne }
    JMP  @NEXT

@MODEM:                            { Changement des signaux modem }
    MOV  DX, [SI+ComX.RegBase]
    ADD  DX, MODEMSTATE
    IN   AL, DX
    MOV  [SI+ComX.ModemSt], AL  { Garde l'etat du modem }
    MOV  [SI+ComX.NModemSt], TRUE   { Signale le nouvel tat }

@NEXT:
    MOV  DX, [SI+ComX.RegBase]
    ADD  DX, ITIDEN
    IN   AL, DX                    { Identification des interruptions }

    TEST AL, 01h                   { Test le Bit 1 }
    JZ   @BOUCLE                   { Saut si interruption en attente }

    LDS  SI, [SI+ComX.Next]        { DS:SI = ComX.Next }
@NEXTCOM0:
    MOV  AX, DS
    OR   AX, SI
    JNZ  @NEXT                     { Saute si DS:SI<>NIL }

    { Sinon, c'est la fin }

    POP  AX

    OR   CL, CL
    JNZ  @PARLA                    { Saut si aucun port n'a exig un traitement }
    XOR  CH, CH                    { sinon, indique au moins UN traitement }
    JMP  @REPART                   { et on recommence }

@PARLA:
    OR   CH, CH
    JZ   @FININT                   { il y a eu au moins un traitement }

                                   { Sinon, il y a peut tre une autre Int avec }
                                   { cet IRQ !!! }

    MOV  SI, AX

    MOV  DX, SEG TabIrq
    MOV  DS, DX

    ADD  SI, OFFSET TabIrq         { DS:SI Pointe sur IrqDef courant }

    CMP  [SI+IrqDef.IrqMsk], FALSE
    JNE  @FININT                   { continue, car l'irq tait masqu! }

    PUSHF
    CALL [SI+IrqDef.IntVec]        { Appeller l'int sauv !! }
    JMP  @FIN                      { Puis sortir }

@FININT:                           { EOI en fonction de l'irq... }
    MOV  AH, AL
    MOV  AL, EOI
    TEST AH, 80h                   { Test le bit 7 }
    JZ   @8259_1                   { Saut si nIrq<8 }
    OUT  0A0h, AL                  { End Of Interrupt sur le second 8259 }
@8259_1:
    OUT  20h, AL                   { End Of Interrupt sur le premier 8259 }

@FIN:
    POP  ES
    POP  DS
    POP  SI
    POP  DX
    POP  CX
    POP  BX

    POP  AX                        { Cas particulier }

    IRET
End;


Procedure Int09h; Far; Assembler;  { Irq 01h }
Asm
    PUSH AX
    MOV  AL, 16          { Decalage dans le tableau TabIrq }
    JMP  IntSerialKernel
End;


Procedure Int0Ah; Far; Assembler;  { Irq 02h }
Asm
    PUSH AX
    MOV  AL, 32
    JMP  IntSerialKernel
End;


Procedure Int0Bh; Far; Assembler;  { Irq 03h }
Asm
    PUSH AX
    MOV  AL, 48
    JMP  IntSerialKernel
End;


Procedure Int0Ch; Far; Assembler;  { Irq 04h }
Asm
    PUSH AX
    MOV  AL, 64
    JMP  IntSerialKernel
End;


Procedure Int0Dh; Far; Assembler;
Asm
    PUSH AX
    MOV  AL, 80
    JMP  IntSerialKernel
End;


Procedure Int0Eh; Far; Assembler;
Asm
    PUSH AX
    MOV  AL, 96
    JMP  IntSerialKernel
End;


Procedure Int0Fh; Far; Assembler;
Asm
    PUSH AX
    MOV  AL, 112
    JMP  IntSerialKernel
End;


Procedure Int70h; Far; Assembler;
Asm
    PUSH AX
    MOV  AL, 128
    JMP  IntSerialKernel
End;


Procedure Int71h; Far; Assembler;
Asm
    PUSH AX
    MOV  AL, 144
    JMP  IntSerialKernel
End;


Procedure Int72h; Far; Assembler;
Asm
    PUSH AX
    MOV  AL, 160
    JMP  IntSerialKernel
End;


Procedure Int73h; Far; Assembler;
Asm
    PUSH AX
    MOV  AL, 176
    JMP  IntSerialKernel
End;


Procedure Int74h; Far; Assembler;
Asm
    PUSH AX
    MOV  AL, 192
    JMP  IntSerialKernel
End;


Procedure Int75h; Far; Assembler;
Asm
    PUSH AX
    MOV  AL, 208
    JMP  IntSerialKernel
End;


Procedure Int76h; Far; Assembler;
Asm
    PUSH AX
    MOV  AL, 224
    JMP  IntSerialKernel
End;


Procedure Int77h; Far; Assembler;
Asm
    PUSH AX
    MOV  AL, 240
    JMP  IntSerialKernel
End;



Const NewInt : Array[1..MaxIrq-1] Of Pointer =
        (@Int09h,@Int0Ah,@Int0Bh,@Int0Ch,@Int0Dh,@Int0Eh,@Int0Fh,
         @Int70h,@Int71h,@Int72h,@Int73h,@Int74h,@Int75h,@Int76h,@Int77h);



  { Irq0..7  Manipulable par les registres $20 et $21 }
  { Irq8..15      "       "   "      "     $A0 et $A1 }

  { L'IRQ 0 EST INUTILISABLE => PAS D'IRQ: CHECKUP Impos }


Function CreateBuffer( Var Buf : BufferRec; Size : Word ) : Boolean;
Begin
  GetMem(Buf.Addr,Size+16);

  Asm
      PUSH DS

      LDS  SI, [Buf]
      MOV  AX, [SI+WORD PTR BufferRec.Addr]
      MOV  DX, [SI+WORD PTR BufferRec.Addr+2]
      OR   AX, DX
      JZ   @FALSE                              { Saut si Nil }

      OR   AX, AX
      JZ   @ICI
      INC  DX
  @ICI:
      MOV  [SI+BufferRec.SegBuf], DX
      MOV  AX, [Size]
      MOV  [SI+BufferRec.Size], AX
      XOR  AX, AX
      MOV  [SI+BufferRec.Read], AX
      MOV  [SI+BufferRec.Write], AX
      MOV  [SI+BufferRec.Nb], AX

      MOV  [@Result], TRUE
      JMP  @FIN

  @FALSE:
      MOV  [@Result], FALSE
  @FIN:
      POP  DS
  End;
End;


Procedure FreeBuffer( Var Buf : BufferRec );
Begin
  If Buf.Addr<>Nil Then FreeMem(Buf.Addr,Buf.Size+16);
End;


Procedure ClearPort( RegBase : Word ); Assembler;
Asm
    MOV  BX, [RegBase]

    MOV  DX, BX
    ADD  DX, ITIDEN
    IN   AL, DX            { Clear ITIDEN }

    MOV  DX, BX
    ADD  DX, LINESTATE
    IN   AL, DX            { Clear LINESTATE }

    MOV  DX, BX
    ADD  DX, MODEMSTATE
    IN   AL, DX            { Clear MODEMSTATE }

    MOV  DX, BX
    IN   AL, DX            { Clear INOUTPUT }
End;


{ Initialise & Ouvre un port avant accs }

Function ComX.Init( RegUART         : Word;
                    NumIrq          : Byte;
                    Baud            : Word;
                    Parameters      : Byte;
                    SizeIn, SizeOut : Word ) : Boolean; Assembler;
Asm
    AND  [RegUART], NOT 7
    JZ   @FALSE               { sort avec FALSE : regUART=0 !! }
    MOV  AL, [NumIrq]
    CMP  AL, MAXIRQ
    JAE  @FALSE               { sort avec FALSE : NumIrq>=MAXIRQ !!! }

    { Com disponible }

    MOV  AX, [SizeIn]
    OR   AX, AX
    JNZ  @_01
    INC  AX                   { 1 => AX }
    JMP  @_02
@_01:
    CMP  AX, InputSizeMax
    JBE  @_02
    MOV  AX, InputSizeMax
@_02:
    MOV  [SizeIn], AX

    MOV  AX, [SizeOut]
    OR   AX, AX
    JNZ  @_03
    INC  AX                   { 1 => AX }
    JMP  @_04
@_03:
    CMP  AX, OutputSizeMax
    JBE  @_04
    MOV  AX, OutputSizeMax
@_04:
    MOV  [SizeOut], AX

    LES  DI, [Self]
    ADD  DI, OFFSET Input
    PUSH ES
    PUSH DI
    PUSH [SizeIn]
    CALL CreateBuffer
    OR   AL, AL
    JZ   @FALSE

    LES  DI, [Self]
    ADD  DI, OFFSET Output
    PUSH ES
    PUSH DI
    PUSH [SizeOut]
    CALL CreateBuffer
    OR   AL, AL
    JNZ  @_05           { saut si OK }

    { librer le buffer d'entre qui a t allou }

    LES  DI, [Self]
    ADD  DI, OFFSET Input
    PUSH ES
    PUSH DI
    CALL FreeBuffer

@FALSE:
    MOV  AL, FALSE
    JMP  @EXIT


@_05:

    PUSH DS

    LDS  SI, Self

    MOV  BX, [RegUART]
    MOV  [SI+RegBase], BX     { RegBase := RegUART(BX) }
    MOV  AL, [NumIrq]
    MOV  [SI+nIrq], AL        { nIrq := NumIrq }
    XOR  AL, AL
    MOV  [SI+Chain], AL       { Chain := FALSE }
    MOV  [SI+DataLost], AL    { DataLost ... }
    MOV  [SI+NLineSt], AL     { NLineSt ... }
    MOV  [SI+NModemSt], AL    { NModemSt ... }

    POP  DS

    { Mettre DLAB=0 et Break=0 }

    MOV  DX, BX               { Rappel BX=RegUART }
    ADD  DX, LINECTRL
    IN   AL, DX
    AND  AL, (DLAB+BREAKON) XOR 255
    OUT  DX, AL

    { Mettre prt en in/out inactive (en attendant) }

    MOV  DX, BX
    ADD  DX, MODEMCTRL
    XOR  AL, AL
    OUT  DX, AL

    { Desactiver toutes les interruptions series }

    MOV  DX, BX
    ADD  DX, ITVALID
    XOR  AL, AL
    OUT  DX, AL

    { 2 petit clear ... }

    PUSH [RegUART]
    CALL ClearPort

    PUSH [RegUART]
    CALL ClearPort



    PUSH DS

    MOV  AL, [NumIrq]
    MOV  CL, 4
    SHL  AL, CL              { AL=0..15 => 0..240 }
    XOR  AH, AH
    MOV  SI, AX

    MOV  AX, SEG TabIrq
    MOV  DS, AX

    ADD  SI, OFFSET TabIrq   { DS:SI Pointe sur IrqDef }

    CLI                      { Interdit les interruptions }

    CMP  [NumIrq], 0
    JE   @PASDIRQ            { Saut si NumIrq=0 }

    MOV  AX, [SI+WORD PTR IrqDef.IntVec]
    OR   AX, [SI+WORD PTR IrqDef.IntVec+2]
    JNZ  @ACTIF              { Saut si IntVec<>Nil }

    MOV  AL, [BYTE PTR NumIrq]
    CMP  AL, 8
    JAE  @AT

    ADD  AL, 08h             { n interruption }
    MOV  CL, 21h             { port 8259 }
    JMP  @01

@AT:
    ADD  AL, 70h-08h         { n interruption }
    MOV  CL, 0A1h            { port 8259 n2 }

@01:
    MOV  AH, 35h
    INT  21h                 { recherche le vecteur interrupt }
                             { resultat dans ES:BX }
    MOV  [SI+WORD PTR IrqDef.IntVec], BX
    MOV  BX, ES
    MOV  [SI+WORD PTR IrqDef.IntVec+2], BX  { note l'adresse }

    MOV  BL, [NumIrq]
    DEC  BL
    SHL  BL, 1
    SHL  BL, 1
    XOR  BH, BH

    MOV  DI, SEG NewInt
    MOV  ES, DI
    MOV  DI, OFFSET NewInt
    ADD  DI, BX              { ES:DI = Pointe sur NewInt }

    PUSH DS                  { Sauve }

    LDS  DX, [ES:DI]         { DS:DX = nouvelle adresse }

    MOV  AH, 25h
    INT  21h                 { fixe la nouvelle adresse }

    POP  DS                  { Restore }


    MOV  DL, CL              { Rappel: CL=port 8259 }
    XOR  DH, DH

    MOV  CL, [NumIrq]
    AND  CL, 7
    MOV  BL, 1
    SHL  BL, CL              { BL = Masque bit interruption }

    IN   AL, DX

    TEST AL, BL
    JZ   @02                 { Interruption NON MASQUE }
    MOV  [SI+IrqDef.IrqMsk], TRUE   { INDIC MASQUE }
    NOT  BL
    AND  AL, BL
    OUT  DX, AL                     { Et demasque }
    JMP  @03
@02:
    MOV  [SI+IrqDef.IrqMsk], FALSE  { NON MASQUE }
@03:

@ACTIF:

    { Valide toutes les interruptions }

    MOV  DX, [RegUART]
    ADD  DX, ITVALID
    MOV  AL, ITINPUT+ITOUTPUT+ITERROR+ITMODEM
    OUT  DX, AL

@PASDIRQ:

    { Fixe la vitesse de communication }

    MOV  BX, [RegUART]
    MOV  DX, BX
    ADD  DX, LINECTRL
    IN   AL, DX
    OR   AL, DLAB
    OUT  DX, AL                  { Mettre DLAB=1 }

    MOV  AX, [Baud]
    MOV  DX, BX
    OUT  DX, AL
    INC  DX
    MOV  AL, AH
    OUT  DX, AL                  { Ecrire la vitesse }

    MOV  DX, BX
    ADD  DX, LINECTRL
    IN   AL, DX
    AND  AL, DLAB XOR 255
    OUT  DX, AL                  { Remettre DLAB=0 }


    { Fixe les parametres de communication }

    MOV  DX, BX
    ADD  DX, LINECTRL
    MOV  AL, [Parameters]
    AND  AL, 3Fh                 { Clr bit 6-7 }
    OUT  DX, AL


    { Mettre prt en in/out }

    MOV  DX, BX
    ADD  DX, MODEMCTRL
    MOV  AL, CT_DTR+CT_RTS
    CMP  [NumIrq], 0
    JE   @03B
    OR   AL, CT_OUT2         { activer l'IRQ du port }
@03B:
    OUT  DX, AL

    PUSH [RegUART]
    CALL ClearPort

    PUSH [RegUART]
    CALL ClearPort

    { Chaine les Com en fonctions de l'irq }

    MOV  AX, [SI+WORD PTR IrqDef.FirstCom]
    OR   AX, [SI+WORD PTR IrqDef.FirstCom+2]
    JNZ  @04         { Saut si FirstCom<>Nil }
    MOV  AX, WORD PTR Self
    MOV  [SI+WORD PTR IrqDef.FirstCom], AX
    MOV  AX, WORD PTR Self+2
    MOV  [SI+WORD PTR IrqDef.FirstCom+2], AX
    JMP  @05
@04:
    LES  DI, [SI+IrqDef.LastCom]
    MOV  AX, WORD PTR Self
    MOV  [ES:DI+WORD PTR ComX.Next], AX
    MOV  AX, WORD PTR Self+2
    MOV  [ES:DI+WORD PTR ComX.Next+2], AX
@05:
    LES  DI, Self
    MOV  AX, [SI+WORD PTR IrqDef.LastCom]
    MOV  [ES:DI+WORD PTR Prec], AX
    MOV  AX, [SI+WORD PTR IrqDef.LastCom+2]
    MOV  [ES:DI+WORD PTR Prec+2], AX

    XOR  AX, AX
    MOV  [ES:DI+WORD PTR Next], AX
    MOV  [ES:DI+WORD PTR Next+2], AX

    MOV  [SI+WORD PTR IrqDef.LastCom], DI
    MOV  AX, ES
    MOV  [SI+WORD PTR IrqDef.LastCom+2], AX

    STI

    POP  DS


    MOV  AL, TRUE      { Ok: on sort avec TRUE }

@EXIT:
End;



{ Ferme la communication et l'objet }

Procedure ComX.Done; Assembler;
Var NumIrq  : Word;
Asm
    LES  DI, Self
    MOV  AL, [ES:DI+nIrq]
    XOR  AH, AH
    MOV  [NumIrq], AX
    MOV  BX, [ES:DI+RegBase] { RegUART => BX }

    CLI

    MOV  DX, BX
    ADD  DX, MODEMCTRL
    XOR  AL, AL
    OUT  DX, AL

    PUSH DS

    MOV  AX, [NumIrq]
    MOV  CL, 4
    SHL  AX, CL              { AX=0..15 => 0..240 }
    MOV  SI, AX

    MOV  AX, SEG TabIrq
    MOV  DS, AX

    ADD  SI, OFFSET TabIrq   { DS:SI Pointe sur IrqDef }

    CMP  [NumIrq], 0
    JE   @PASDIRQ

    { Mettre DLAB=0 }

    MOV  DX, BX              { Rappel BX=RegUART }
    ADD  DX, LINECTRL
    IN   AL, DX
    AND  AL, DLAB XOR 255
    OUT  DX, AL

    { Arrete toutes les interruptions serie }

    MOV  DX, BX              { Rappel BX=RegUART }
    ADD  DX, ITVALID
    XOR  AL, AL
    OUT  DX, AL

    MOV  AX, [SI+WORD PTR IrqDef.FirstCom+2]
    CMP  AX, [SI+WORD PTR IrqDef.LastCom+2]
    JNE  @PASDERNIER
    MOV  AX, [SI+WORD PTR IrqDef.FirstCom]
    CMP  AX, [SI+WORD PTR IrqDef.LastCom]
    JNE  @PASDERNIER

    { C'est le dernier Object sur cet Irq: }

    MOV  AL, [BYTE PTR NumIrq]
    CMP  AL, 8
    JAE  @AT

    ADD  AL, 08h             { n interruption }
    MOV  CL, 21h             { port 8259 }
    JMP  @01

@AT:
    ADD  AL, 70h-08h         { n interruption }
    MOV  CL, 0A1h            { port 8259 n2 }

@01:
    { Restore l'adresse d'origine }

    PUSH DS                  { Sauve }

    MOV  AH, 25h
    LDS  DX, [SI+IrqDef.IntVec]
    INT  21h

    POP  DS                  { Restore }

    CMP  [SI+IrqDef.IrqMsk], FALSE
    JE   @02

    { Interdit l'interruption }

    MOV  DL, CL
    XOR  DH, DH

    MOV  CX, [NumIrq]
    AND  CL, 7
    MOV  BL, 1
    SHL  BL, CL              { BL = Masque bit interruption }

    IN   AL, DX
    OR   AL, BL
    OUT  DX, AL              { on remasque l'interruption }

@02:
    XOR  AX, AX
    MOV  [SI+WORD PTR IrqDef.IntVec], AX
    MOV  [SI+WORD PTR IrqDef.IntVec+2], AX

@PASDERNIER:

@PASDIRQ:

    MOV  AX, DS
    MOV  ES, AX
    MOV  DI, SI              { DS:SI => ES:DI }

    LDS  SI, Self            { DS:SI = Self }

    MOV  AX, [SI+WORD PTR Next]
    OR   AX, [SI+WORD PTR Next+2]
    JZ   @03                 { Saut si Next=Nil }

    PUSH ES                  { Sauve }

    LES  BX, [SI+Next]       { ES:BX = Next^ }
    MOV  AX, [SI+WORD PTR Prec]
    MOV  [ES:BX+WORD PTR ComX.Prec], AX
    MOV  AX, [SI+WORD PTR Prec+2]
    MOV  [ES:BX+WORD PTR ComX.Prec+2], AX  { Next^.Prec := Prec }

    POP  ES                  { restore }
    JMP  @04
@03:
    MOV  AX, [SI+WORD PTR Prec]
    MOV  [ES:DI+WORD PTR IrqDef.LastCom], AX
    MOV  AX, [SI+WORD PTR Prec+2]
    MOV  [ES:DI+WORD PTR IrqDef.LastCom+2], AX
@04:
    MOV  AX, [SI+WORD PTR Prec]
    OR   AX, [SI+WORD PTR Prec+2]
    JZ   @05                 { Saut si Prec=Nil }

    PUSH ES                  { Sauve }

    LES  BX, [SI+Prec]               { ES:BX = Prec^ }
    MOV  AX, [SI+WORD PTR Next]
    MOV  [ES:BX+WORD PTR ComX.Next], AX
    MOV  AX, [SI+WORD PTR Next+2]
    MOV  [ES:BX+WORD PTR ComX.Next+2], AX  { Prec^.Next := Next }

    POP  ES                  { restore }
    JMP  @06
@05:
    MOV  AX, [SI+WORD PTR Next]
    MOV  [ES:DI+WORD PTR IrqDef.FirstCom], AX
    MOV  AX, [SI+WORD PTR Next+2]
    MOV  [ES:DI+WORD PTR IrqDef.FirstCom+2], AX
@06:
    POP  DS

    STI


    LES  DI, [Self]
    ADD  DI, OFFSET Output
    PUSH ES
    PUSH DI
    CALL FreeBuffer

    LES  DI, [Self]
    ADD  DI, OFFSET Input
    PUSH ES
    PUSH DI
    CALL FreeBuffer
End;


{ Fixe la vitesse de communication }

Procedure ComX.SetBaud( Baud : Word ); Assembler;
Asm
    PUSH DS
    LDS  SI, Self
    MOV  BX, [SI+RegBase]
    MOV  DX, BX
    ADD  DX, LINECTRL

    CLI                          { Car on va mettre DLAB=1 }

    IN   AL, DX
    OR   AL, DLAB
    OUT  DX, AL                  { Mettre DLAB=1 }

    MOV  AX, [Baud]
    MOV  DX, BX
    OUT  DX, AL
    INC  DX
    MOV  AL, AH
    OUT  DX, AL                  { Ecrire la vitesse }

    MOV  DX, BX
    ADD  DX, LINECTRL
    IN   AL, DX
    AND  AL, DLAB XOR 255
    OUT  DX, AL                  { Mettre DLAB=0 }

    STI

    POP  DS
End;


{ Fixe les parametres de communication }

Procedure ComX.SetParam( Parameters : Byte ); Assembler;
Asm
    LES  DI, Self
    MOV  DX, [ES:DI+RegBase]
    ADD  DX, LINECTRL
    MOV  AL, [Parameters]
    AND  AL, 3Fh                 { Clr bit 6-7 }
    OUT  DX, AL
End;


{ Fixe l'etat Break }

Procedure ComX.SetBreak( BrkOn : Boolean ); Assembler;
Asm
    LES  DI, Self
    MOV  DX, [ES:DI+RegBase]
    ADD  DX, LINECTRL
    IN   AL, DX                  { Lire l'etat de la ligne }
    CMP  [BrkOn], FALSE
    JE   @OFF
    OR   AL, BREAKON
    JMP  @ICI
@OFF:
    AND  AL, BREAKON XOR 255
@ICI:
    OUT  DX, AL
End;


{ Fixe l'etat LOOP }

Procedure ComX.SetLoop( LoopOn : Boolean ); Assembler;
Asm
    LES  DI, Self
    MOV  DX, [ES:DI+RegBase]
    ADD  DX, MODEMCTRL
    IN   AL, DX                  { Lire l'etat de la ligne }
    CMP  [LoopOn], FALSE
    JE   @OFF
    OR   AL, CTRL_LOOP
    JMP  @ICI
@OFF:
    AND  AL, CTRL_LOOP XOR 255
@ICI:
    OUT  DX, AL
End;


{ Controle du modem }

Procedure ComX.SetModemCtrl( Parameters : Byte ); Assembler;
Asm
    LES  DI, Self
    MOV  DX, [ES:DI+RegBase]
    ADD  DX, MODEMCTRL
    IN   AL, DX                  { Lire l'etat de la ligne }
    AND  AL, (CT_DTR+CT_RTS+CT_OUT1+CT_OUT2) XOR 255  { garde bits 4  7 }
    MOV  BL, [Parameters]
    AND  BL, CT_DTR+CT_RTS+CT_OUT1+CT_OUT2     { garde bits 0  3 }
    OR   AL, BL
    OUT  DX, AL                    { Ecrire }
End;


{ Lire le controle du modem }

Function  ComX.GetModemCtrl : Byte; Assembler;
Asm
    LES  DI, Self
    MOV  DX, [ES:DI+RegBase]
    ADD  DX, MODEMCTRL
    IN   AL, DX                  { Lire l'etat de la ligne }
    AND  AL, CT_DTR+CT_RTS+CT_OUT1+CT_OUT2  { clear bits 4  7 }
End;



{ Renvoi TRUE si l'etat du modem a chang }
{ (Remis  zro (FALSE) aprs lecture de l'etat du modem ) }

Function  ComX.NewModemState : Boolean; Assembler;
Asm
    PUSH DS
    LDS  SI, Self
    CLI
    CMP  [SI+NModemSt], FALSE
    JE   @LA                         { Saut si pas de changement }
    MOV  AL, [SI+ModemSt]            { Charge l'etat sauv }
    JMP  @ICI
@LA:
    MOV  DX, [SI+RegBase]
    ADD  DX, MODEMSTATE
    IN   AL, DX                      { Lire le status du modem }
    MOV  [SI+NModemSt], TRUE
    MOV  [SI+ModemSt], AL            { Sauve l'etat lu }
@ICI:
    STI
    AND  AL, DL_RTS+DL_DSR+DL_RI+DL_DCD
    JZ   @FIN                        { Sort avec FALSE }
    MOV  AL, TRUE                    { Changement modem : renvoi TRUE }
@FIN:
    POP  DS
End;


{ Renvoi l'etat du modem }

Function  ComX.GetModemState : Byte; Assembler;
Asm
    PUSH DS
    LDS  SI, Self
    CLI
    CMP  [SI+NModemSt], FALSE
    JE   @LA                         { Saut si pas de changement }
    MOV  AL, [SI+ModemSt]            { Charge l'etat sauv }
    MOV  [SI+NModemSt], FALSE
    JMP  @FIN
@LA:
    MOV  DX, [SI+RegBase]
    ADD  DX, MODEMSTATE
    IN   AL, DX                      { Lire le status du modem }
@FIN:
    STI
    POP  DS
End;



{ Renvoi TRUE si des erreurs existent sur la ligne }

Function ComX.LineError : Boolean; Assembler;
Asm
    PUSH DS
    LDS  SI, Self
    CLI
    MOV  DX, [SI+RegBase]
    ADD  DX, LINESTATE
    IN   AL, DX                      { Lire le status de la ligne }
    CMP  [SI+NLineSt], FALSE
    JE   @LA
    MOV  BL, [SI+LineSt]
    AND  BL, LN_ERROR                { garder les bits d'erreurs }
    OR   AL, BL                      { Ajouter les erreurs eventuelles }
@LA:
    CMP  [SI+DataLost], FALSE
    JE   @ICI                        { saut si pas de dbordement }
    OR   AL, LN_OVRN                 { signale le debordement }
@ICI:
    MOV  [SI+NLineSt], TRUE          { c'est obligatoirement lu ... }
    MOV  [SI+LineSt], AL             { conserve le rsultat }
    AND  AL, LN_ERROR                { tester les bits erreurs }
    JZ   @FIN                        { pas d'erreur: renvoi FALSE }
    MOV  AL, TRUE
@FIN:
    STI
    POP  DS
End;


{ Renvoi l'etat de la ligne }
{ (Remise  zro aprs lecture de l'etat de la ligne) }

Function  ComX.GetLineState : Byte; Assembler;
Asm
    PUSH DS
    LDS  SI, Self
    CLI
    MOV  DX, [SI+RegBase]
    ADD  DX, LINESTATE
    IN   AL, DX                      { Lire le status de la ligne }
    CMP  [SI+NLineSt], FALSE
    JE   @LA                         { Saut si pas d'erreurs signal }

    MOV  [SI+NLineSt], FALSE
    MOV  AH, [SI+LineSt]             { Lire le status sauv }
    AND  AH, LN_ERROR                { ne garder que les bits erreurs }
    OR   AL, AH                      { les integrer dans le status rel }
@LA:
    CMP  [SI+DataLost], FALSE
    JE   @FIN                        { Saut si pas de dbordement }
    OR   AL, LN_OVRN                 { Signaler un dbordement }
    MOV  [SI+DataLost], FALSE
@FIN:
    STI
    POP  DS
End;



{ Renvoi le nombre de donnes  lire }

Function  ComX.ReceiveCount : Word; Assembler;
Asm
    LES  DI, Self
    MOV  AX, [ES:DI+Input.Nb]
End;


{ Renvoi la place libre dans le tampon entre }

Function  ComX.ReceiveAvail : Word; Assembler;
Asm
    LES  DI, Self
    MOV  AX, [ES:DI+Input.Size]
    SUB  AX, [ES:DI+Input.Nb]
End;


{ Lire un caractere }

Function  ComX.ReceiveChar( Var C : Char ) : Boolean; Assembler;
Asm
    PUSH DS
    LDS  SI, Self
    CMP  [SI+Input.Nb], 0
    JE   @RIEN
    MOV  ES, [SI+Input.SegBuf]
    MOV  DI, [SI+Input.Read]
    MOV  AL, [ES:DI]                { lit la donne }
    INC  DI
    CMP  DI, [SI+Input.Size]
    JB   @ICI
    XOR  DI, DI
@ICI:
    MOV  [SI+Input.Read], DI
    DEC  [SI+Input.Nb]
    LES  DI, C                      { Charge pointer sur "C" }
    MOV  [ES:DI], AL                { Ecrit dans "C" }
    MOV  AL, TRUE                   { Renvoi TRUE }
    JMP  @FIN
@RIEN:
    XOR  AL, AL                     { Renvoi FALSE }
@FIN:
    POP  DS
End;


{ Charge une ou plusieurs donnes en attente }
{ Renvoi le nombre de caractre lu }

Function  ComX.Receive( Var P; LgMax : Word ) : Word; Assembler;
Asm
    PUSH DS
    LDS  SI, Self
    XOR  BX, BX                     { Compteur de donnes extraites }
    JMP  @TEST
@BOUCLE:
    MOV  ES, [SI+ComX.Input.SegBuf]
    MOV  DI, [SI+ComX.Input.Read]
    MOV  AL, [ES:DI]                { lit la donne }
    INC  DI
    CMP  DI, [SI+ComX.Input.Size]
    JB   @ICI
    XOR  DI, DI
@ICI:
    MOV  [SI+ComX.Input.Read], DI
    DEC  [SI+ComX.Input.Nb]
    LES  DI, P                      { Charge pointer sur "C" }
    MOV  [ES:DI+BX], AL
    INC  BX
@TEST:
    CMP  BX, [LgMax]
    JAE  @LIMITE                    { Saut si BX>=LgMax }
    CMP  [SI+ComX.Input.Nb], 0
    JNE  @BOUCLE                    { Saut si encore  lire ... }
@LIMITE:
    MOV  AX, BX                     { Place le rsultat }
    POP  DS
End;


{ Supprime toutes les donnes restant dans le Buffer IN }

Procedure ComX.ReceiveClear; Assembler;
Asm
    PUSH DS
    LDS  SI, Self
    XOR  AX, AX
    CLI
    MOV  [SI+Input.Read], AX
    MOV  [SI+Input.Write], AX
    MOV  [SI+Input.Nb], AX
    STI
    POP  DS
End;


{ Renvoi le nombre de donnes restant  emettre }

Function  ComX.TransmitCount : Word; Assembler;
Asm
    LES  DI, Self
    MOV  AX, [ES:DI+Output.Nb]
End;


{ Renvoi la place libre dans le tampon sortie }

Function  ComX.TransmitAvail : Word; Assembler;
Asm
    LES  DI, Self
    MOV  AX, [ES:DI+Output.Size]
    SUB  AX, [ES:DI+Output.Nb]
End;


{ Ecrit un caractre }

Function  ComX.TransmitChar( C : Char ) : Boolean; Assembler;
Asm
    PUSH DS
    LDS  SI, Self
    MOV  AX, [SI+Output.Size]
    CMP  AX, [SI+Output.Nb]
    JBE  @PLEIN                     { Saut si pas de place }

    MOV  ES, [SI+Output.SegBuf]     { Mettre la donne dans le buffer }
    MOV  DI, [SI+Output.Write]
    MOV  AL, [C]
    MOV  [ES:DI], AL                { place la donne dans le tampon }
    INC  DI
    CMP  DI, [SI+Output.Size]
    JB   @01
    XOR  DI, DI
@01:
    MOV  [SI+Output.Write], DI
    INC  [SI+Output.Nb]

    CMP  [SI+Chain], FALSE
    JNE  @OK                        { Saut si chainage en cours }

    CMP  [SI+Output.Nb], 0
    JE   @OK                        { Saut si buffer dj vide !!! }

    MOV  DX, [SI+RegBase]
    ADD  DX, LINESTATE
    IN   AL, DX                     { Lire l'etat de la ligne }
    TEST AL, LN_THRE
    JZ   @ABORT                     { Sortie non disponible }

    MOV  ES, [SI+Output.SegBuf]     { sinon, Chainage  lancer }
    MOV  DI, [SI+Output.Read]
    MOV  AL, [ES:DI]
    INC  DI
    CMP  DI, [SI+Output.Size]
    JB   @02
    XOR  DI, DI
@02:
    MOV  [SI+Output.Read], DI
    DEC  [SI+Output.Nb]
    MOV  DX, [SI+RegBase]
    CLI                             { CLI/STI a cause de "CheckUp" }
    MOV  [SI+Chain], TRUE           { Annonce le chainage }
    OUT  DX, AL                     { Lance le chainage (envoi la donne) }
    STI
    JMP  @OK                        { Renvoi TRUE }
@ABORT:                             { Abandonn: sortie non prete }
    MOV  AX, [SI+Output.Write]
    OR   AX, AX
    JNZ  @04
    MOV  AX, [SI+Output.Size]
@04:
    DEC  AX
    MOV  [SI+Output.Write], AX
    DEC  [SI+Output.Nb]
@PLEIN:
    XOR  AL, AL                     { Sort avec FALSE: caractere non crit }
    JMP  @FIN
@OK:
    MOV  AL, TRUE
@FIN:
    POP  DS
End;


{ Envoi un paquet de donne }

Function  ComX.Transmit( Var p; Lg : Word ) : Word; Assembler;
Asm
    PUSH DS
    LDS  SI, Self
    XOR  BX, BX
    MOV  CX, [Lg]
    JCXZ @CPLEIN
    CLD                             { car on use de "STOSB" }
@BOUCLE:
    MOV  AX, [SI+Output.Size]
    CMP  AX, [SI+Output.Nb]
    JBE  @CPLEIN
    LES  DI, [P]
    MOV  AL, [ES:DI+BX]

    MOV  ES, [SI+Output.SegBuf]
    MOV  DI, [SI+Output.Write]
    STOSB
    CMP  DI, [SI+Output.Size]
    JB   @01
    XOR  DI, DI
@01:
    MOV  [SI+Output.Write], DI
    INC  [SI+Output.Nb]
    INC  BX
    LOOP @BOUCLE
@CPLEIN:

    CMP  [SI+Chain], FALSE
    JNE  @03                        { Saut si chainage en cours }

    CMP  [SI+Output.Nb], 0
    JE   @03                        { Saut si buffer dj vide !!! }

    MOV  DX, [SI+RegBase]
    ADD  DX, LINESTATE
    IN   AL, DX                     { Lire l'etat de la ligne }
    TEST AL, LN_THRE
    JZ   @ABORT                     { Sortie non disponible }

    MOV  ES, [SI+Output.SegBuf]     { sinon, Chainage  lancer }
    MOV  DI, [SI+Output.Read]
    MOV  AL, [ES:DI]
    INC  DI
    CMP  DI, [SI+Output.Size]
    JB   @02
    XOR  DI, DI
@02:
    MOV  [SI+Output.Read], DI
    DEC  [SI+Output.Nb]
    MOV  DX, [SI+RegBase]
    CLI                             { CLI/STI a cause de "CheckUp" }
    MOV  [SI+Chain], TRUE           { Annonce le chainage }
    OUT  DX, AL                     { Lance le chainage (envoi la donne) }
    STI
@03:
    MOV  AL, TRUE
    JMP  @FIN
@ABORT:                             { Abandonn: sortie non prete }
    OR   CX, CX
    JNZ  @FIN                       { Saut si tout n'est pas emit }
    MOV  AX, [SI+Output.Write]      { sinon, on sort le dernier car }
    OR   AX, AX                     { (car il faudra relancer) }
    JNZ  @04
    MOV  AX, [SI+Output.Size]
@04:
    DEC  AX
    MOV  [SI+Output.Write], AX
    DEC  [SI+Output.Nb]
    DEC  BX
@FIN:
    MOV  AX, BX                     { Renvoi le nombre install }
    POP  DS
End;


{ Supprime toutes les donnes restant dans le Buffer OUT }

Procedure ComX.TransmitClear; Assembler;
Asm
    PUSH DS
    LDS  SI, Self
    XOR  AX, AX
    CLI
    MOV  [SI+Output.Read], AX
    MOV  [SI+Output.Write], AX
    MOV  [SI+Output.Nb], AX
    STI
    POP  DS
End;



{ Installe une interruption pour le CheckUp sur XT }

Const NOCHECKUP = $FF;      { Flag pour SpeedXT et SpeedAT }

Var OrgPhase08 : Word;
    Phase08    : Word;
    ExInt08    : Pointer;
    SpeedXT    : Byte;

Procedure IntCheckUpXT; Far; Assembler;
Asm
    PUSH AX
    PUSH DS

    MOV  AX, SEG @DATA
    MOV  DS, AX

    DEC  [Phase08]
    JNZ  @EOI

    PUSHF
    CALL [ExInt08]

    MOV  AX, [OrgPhase08]
    MOV  [Phase08], AX
    JMP  @FIN

@EOI:
    MOV  AL, EOI
    OUT  20h, AL

@FIN:
    CALL RanCheckUp

    POP  DS
    POP  AX

    IRET
End;



Procedure Rest_ExInt08; Far; Assembler;
Asm
    PUSH AX
    PUSH DX
    PUSH DS

    MOV  AX, SEG @DATA
    MOV  DS, AX

    DEC  [Phase08]
    JNZ  @LA                    { il faut attendre }

    PUSHF
    CALL [ExInt08]

    MOV  AX, [OrgPhase08]

    OR   AX, AX
    JZ   @REST                  { c'est la fin }

    MOV  [Phase08], AX
    JMP  @01
@LA:
    MOV  AL, EOI
    OUT  20h, AL
@01:
    CMP  [Phase08], 1
    JNE  @FIN

    MOV  [OrgPhase08], 0

    MOV  AL, 34h
    OUT  43h, AL
    MOV  AL, 00h
    OUT  40h, AL
    OUT  40h, AL

    JMP  @FIN

@REST:
    MOV  [SpeedXT], NOCHECKUP

    MOV  AX, 2508h
    LDS  DX, [ExInt08]
    INT  21h

@FIN:
    POP  DS
    POP  DX
    POP  AX

    IRET
End;


{ Renvoi FALSE si dj install }

Function InstCheckUp( Speed : Byte ) : Boolean; Assembler;
Asm
    CMP  [SpeedXT], NOCHECKUP
    JE   @CONTINUE                   { Saute si pas dj install }
    XOR  AL, AL
    JMP  @FIN                        { Sort avec FALSE }

@CONTINUE:
    MOV  AX, 3508h
    INT  21h
    MOV  [WORD PTR ExInt08], BX
    MOV  [WORD PTR ExInt08+2], ES    { Sauvegarde }

    MOV  CL, [Speed]
    AND  CL, 0Fh
    MOV  AX, 1
    SHL  AX, CL                      { Nombre de fois plus rapide }

    MOV  [OrgPhase08], AX

    XOR  AX, AX
    STC
    RCR  AX, CL

    MOV  DX, AX                      { Valeur  ecrire dans le compteur }

    MOV  [Phase08], 1                { Normal }

    CLI

    MOV  AL, [Speed]
    AND  AL, 0Fh
    MOV  [SpeedXT], AL               { garde la vitesse ... }

    MOV  AL, 34h
    OUT  43h, AL
    MOV  AL, DL
    OUT  40h, AL
    MOV  AL, DH
    OUT  40h, AL                     { Arme le compteur }

    PUSH DS

    MOV  AX, 2508h
    MOV  DX, SEG IntCheckUpXT
    MOV  DS, DX
    MOV  DX, OFFSET IntCheckUpXT
    INT  21h

    POP  DS

    STI

    MOV  AL, TRUE
@FIN:
End;


{ Vire l'installation du CheckUp }

Procedure KillCheckUp; Assembler;
Asm
    CMP  [SpeedXT], NOCHECKUP
    JE   @FIN

    PUSH DS
    MOV  AX, 2508h
    MOV  DX, SEG Rest_ExInt08
    MOV  DS, DX
    MOV  DX, OFFSET Rest_ExInt08
    INT  21h                         { mettre sur la desinstallation }
    POP  DS

@BOUCLE:
    CMP  [SpeedXT], NOCHECKUP
    JNE  @BOUCLE                     { attendre la desinstallation }
@FIN:
End;


{ Installation du CheckUp en fonction de l'horloge des AT }

Var ExSpeed  : Byte;
    ExEtat   : Byte;
    ExInt70  : Pointer;
    SpeedAT  : Boolean;
    Irq08Msk : Boolean;

Procedure IntCheckUpAT; Far; Assembler;
Asm
    PUSH AX
    PUSH DX
    PUSH DS

    MOV  AL, 0Ch
    OUT  70h, AL
    IN   AL, 71h                   { cette lecture est obligatoire }
    TEST AL, 01000000b
    JZ   @FIN

    CALL RanCheckUp

@FIN:
    MOV  AL, EOI
    OUT  0A0h, AL
    OUT  020h, AL

    POP  DS
    POP  DX
    POP  AX

    IRET
End;


{ renvoi FALSE si dj utilis, ou si l'horloge ne semble pas libre }

Function InstCheckUpAT( Speed : Byte ) : Boolean; Assembler;
Asm
    CMP  [SpeedAT], NOCHECKUP
    JNE  @FALSE
    IN   AL, 0A1h
    AND  AL, 1
    MOV  [Irq08Msk], AL      { note que l'etat etait masqu ou non }
    JNZ  @CONTINUE           { saut si masqu }
    CLI
    MOV  AL, 0Bh
    OUT  70h, AL
    IN   AL, 71h             { voir les interruptions en cours ? }
    STI
    TEST AL, 01110000b
    JZ   @CONTINUE           { saut si aucune interruption active }

    { sinon, il y a une (ou plusieurs) interruptions en cours }
@FALSE:
    XOR  AL, AL
    JMP  @FIN

@CONTINUE:
    CLI

    MOV  AX, 3570h                   { sauve adresse de l'int 70h !!! }
    INT  21h
    MOV  [WORD PTR ExInt70], BX
    MOV  [WORD PTR ExInt70+2], ES

    PUSH DS

    MOV  AX, 2570h
    MOV  DX, SEG IntCheckUpAT
    MOV  DS, DX
    MOV  DX, OFFSET IntCheckUpAT
    INT  21h                         { nouvelle interruption }

    POP  DS

    MOV  AL, [Speed]
    AND  AL, 0Fh
    MOV  [SpeedAT], AL               { mmorise la vitesse }

    MOV  DL, [Speed]
    NOT  DL
    AND  DL, 00001111b

    MOV  AL, 0Ah
    MOV  AH, AL                      { prev. mauvais compatible }
    OUT  70h, AL
    IN   AL, 71h
    MOV  [ExSpeed], AL               { Sauve la frequence courant }
    AND  AL, 01110000b               { Annule les bits gnants }
    OR   AL, DL
    XCHG AH, AL                      { prev. }
    OUT  70h, AL                     { prev. rappel registre a modifier }
    XCHG AH, AL                      { prev. }
    OUT  71h, AL                     { Fixer la nouvelle vitesse }

    MOV  AL, 0Bh
    MOV  AH, AL                      { prev. }
    OUT  70h, AL
    IN   AL, 71h
    MOV  [ExEtat], AL                { Sauve l'etat }
    AND  AL, 10001111b               { Clr les bits d'interruptions }
    OR   AL, 01000000b
    XCHG AH, AL                      { prev. }
    OUT  70h, AL                     { prev. }
    XCHG AH, AL                      { prev. }
    OUT  71h, AL                     { Autoriser les interruptions periodiques }

    IN   AL, 0A1h
    AND  AL, 11111110b               { Autoriser l'interruption Horloge AT }
    OUT  0A1h, AL

    STI

    MOV  AL, TRUE
@FIN:
End;


{ Procedure virant le CheckUp AT }

Procedure KillCheckUpAT; Assembler;
Asm
    CMP  [SpeedAT], NOCHECKUP
    JE   @FIN
    CLI

    CMP  [Irq08Msk], FALSE
    JE   @01                         { saut si l'interruption n'etait }
                                     { pas masqu }
    IN   AL, 0A1h
    OR   AL, 00000001b
    OUT  0A1h, AL                    { Remasquer l'interruption }
@01:
    MOV  DL, [ExEtat]
    AND  DL, 01110000b

    MOV  AL, 0Bh
    MOV  AH, AL                      { prev. }
    OUT  70h, AL
    IN   AL, 71h
    AND  AL, 10001111b
    OR   AL, DL
    XCHG AH, AL                      { prev. }
    OUT  70h, AL                     { prev. }
    XCHG AH, AL                      { prev. }
    OUT  71h, AL                     { Restore l'etat sauv }

    MOV  DL, [ExSpeed]
    AND  DL, 00001111b

    MOV  AL, 0Ah
    MOV  AH, AL                      { prev. }
    OUT  70h, AL
    IN   AL, 71h
    AND  AL, 11110000b
    OR   AL, DL
    XCHG AH, AL                      { prev. }
    OUT  70h, AL                     { prev. }
    XCHG AH, AL                      { prev. }
    OUT  71h, AL                     { restore la vitesse }

    PUSH DS

    MOV  AX, 2570h
    LDS  DX, [ExInt70]
    INT  21h                         { restore le vecteur }

    POP  DS

    MOV  [SpeedAT], NOCHECKUP

    STI
@FIN:
End;


Var XSerIsClose : Boolean;

Procedure CloseAll; Assembler;
Asm
    CMP  [XSerIsClose], FALSE
    JNE  @EXIT                          { On ne peut fermer + de 1 fois !!! }

    PUSH DS                             { sauve }

    MOV  AX, SEG TabIrq
    MOV  DS, AX
    MOV  SI, OFFSET TabIrq

    XOR  CX, CX

@BOUCLE:
    PUSH SI
    PUSH DS

    CLI

    LDS  SI, [SI+IrqDef.FirstCom]
    JMP  @VER

@BOUCLE_A:
    MOV  BX, [SI+ComX.RegBase]
    MOV  DX, BX
    ADD  DX, MODEMCTRL
    IN   AL, DX
    MOV  [SI+ComX.KpMdCtrl], AL     { Sauve }
    XOR  AL, AL
    OUT  DX, AL

    { Mettre DLAB=0 }
    MOV  DX, BX
    ADD  DX, LINECTRL
    IN   AL, DX
    AND  AL, DLAB XOR 255
    OUT  DX, AL

    { Arrete toutes les interruption }
    MOV  DX, BX
    ADD  DX, ITVALID
    IN   AL, DX
    MOV  [SI+ComX.KpITValid], AL    { Sauve }
    XOR  AL, AL
    OUT  DX, AL

    LDS  SI, [SI+ComX.Next]
@VER:
    MOV  AX, DS
    OR   AX, SI
    JNZ  @BOUCLE_A

    POP  DS
    POP  SI

    MOV  AX, [SI+WORD PTR IrqDef.IntVec]
    OR   AX, [SI+WORD PTR IrqDef.IntVec+2]
    JZ   @RIEN

    MOV  AL, CL
    CMP  AL, 8
    JAE  @AT

    ADD  AL, 08h             { n interruption }
    MOV  DL, 21h             { port 8259 }
    JMP  @01

@AT:
    ADD  AL, 70h-08h         { n interruption }
    MOV  DL, 0A1h            { port 8259 n2 }

@01:
    { Restore l'adresse d'origine }

    PUSH DX
    PUSH DS                  { Sauve }

    MOV  AH, 25h
    LDS  DX, [SI+IrqDef.IntVec]
    INT  21h

    POP  DS                  { Restore }
    POP  DX

    CMP  [SI+IrqDef.IrqMsk], FALSE
    JE   @02

    { Interdit l'interruption }

    XOR  DH, DH

    MOV  AX, CX              { sauve CX dans AX }

    AND  CL, 7
    MOV  BL, 1
    SHL  BL, CL              { BL = Masque bit interruption }

    MOV  CX, AX              { restore CX }

    IN   AL, DX
    OR   AL, BL
    OUT  DX, AL              { on remasque l'interruption }

@02:
    { L'Irq n'est plus utilis }
    { Mais on laisse l'adresse ... }

@RIEN:
    STI

    ADD  SI, 16
    INC  CX
    CMP  CX, MAXIRQ
    JB   @BOUCLE

    POP  DS                            { restore }

    MOV  [XSerIsClose], TRUE

@EXIT:
End;


Procedure OpenAll; Assembler;
Asm
    CMP  [XSerIsClose], FALSE
    JE   @EXIT                          { La com n'est pas ferme !!! }

    PUSH DS                             { sauve }

    MOV  AX, SEG TabIrq
    MOV  DS, AX
    MOV  SI, OFFSET TabIrq

    XOR  CX, CX

@BOUCLE:
    PUSH SI
    PUSH DS

    CLI

    LDS  SI, [SI+IrqDef.FirstCom]
    JMP  @VER

@BOUCLE_A:
    MOV  BX, [SI+ComX.RegBase]
    MOV  DX, BX
    ADD  DX, MODEMCTRL
    MOV  AL, [SI+ComX.KpMdCtrl]     { Retablit l'etat du controleur modem }
    OUT  DX, AL

    { Mettre DLAB=0 }
    MOV  DX, BX
    ADD  DX, LINECTRL
    IN   AL, DX
    AND  AL, DLAB XOR 255
    OUT  DX, AL

    { Retablit les interruptions }
    MOV  DX, BX
    ADD  DX, ITVALID
    MOV  AL, [SI+ComX.KpITValid]
    OUT  DX, AL

    LDS  SI, [SI+ComX.Next]
@VER:
    MOV  AX, DS
    OR   AX, SI
    JNZ  @BOUCLE_A

    POP  DS
    POP  SI

    MOV  AX, [SI+WORD PTR IrqDef.IntVec]
    OR   AX, [SI+WORD PTR IrqDef.IntVec+2]
    JZ   @RIEN

    MOV  AL, CL
    CMP  AL, 8
    JAE  @AT

    ADD  AL, 08h             { n interruption }
    MOV  DL, 21h             { port 8259 }
    JMP  @01

@AT:
    ADD  AL, 70h-08h         { n interruption }
    MOV  DL, 0A1h            { port 8259 n2 }

@01:
    { Remettre l'adresse de "nouvelle" int }

    PUSH DX                  { Sauve }

    MOV  BL, CL              { CL=nIRQ }
    DEC  BL
    SHL  BL, 1
    SHL  BL, 1
    XOR  BH, BH

    MOV  DI, SEG NewInt
    MOV  ES, DI
    MOV  DI, OFFSET NewInt
    ADD  DI, BX              { ES:DI = Pointe sur NewInt }

    PUSH DS                  { Sauve }

    LDS  DX, [ES:DI]         { DS:DX = nouvelle adresse }

    MOV  AH, 25h
    INT  21h                 { restore la nouvelle adresse }

    POP  DS                  { Restore }

    POP  DX                  { Restore }

    CMP  [SI+IrqDef.IrqMsk], FALSE
    JE   @02

    { Autoriser l'interruption ncessaire }

    XOR  DH, DH

    MOV  AX, CX              { sauve CX dans AX }

    AND  CL, 7
    MOV  BL, 1
    SHL  BL, CL              { BL = Masque bit interruption }

    MOV  CX, AX              { restore CX }

    IN   AL, DX
    NOT  BL
    AND  AL, BL
    OUT  DX, AL              { on dmasque l'interruption }

@02:

@RIEN:
    STI

    ADD  SI, 16
    INC  CX
    CMP  CX, MAXIRQ
    JB   @BOUCLE

    POP  DS                            { restore }

    MOV  [XSerIsClose], FALSE

@EXIT:
End;


Var KeepSpeedXT : Byte;
    KeepSpeedAT : Byte;

{ Deconnecter le noyau de communication }

Procedure KeepXSer; Assembler;
Asm
    MOV  AL, [SpeedXT]
    MOV  [KeepSpeedXT], AL      { sauve la vitesse }
    CALL KillCheckUp            { vire le CheckUp XT  (si existe) }

    MOV  AL, [SpeedAT]
    MOV  [KeepSpeedAT], AL      { sauve la vitesse }
    CALL KillCheckUpAT          { vire le CheckUp AT  (si existe) }

    { On va remettre toutes les Int/IRQ detourns }

    CALL CloseAll   { On ferme toutes les coms, au niveau matriel }
End;


{ On Re-connecte le noyau de communication }

Procedure ResumeXSer; Assembler;
Asm
    CALL OpenAll    { On (Re-)ouvre toutes les communications }

    MOV  AL, [KeepSpeedAT]
    CMP  AL, NOCHECKUP
    JE   @01
    PUSH AX
    CALL InstCheckUpAT
@01:
    MOV  AL, [KeepSpeedXT]
    CMP  AL, NOCHECKUP
    JE   @02
    PUSH AX
    CALL InstCheckUp
@02:
End;


{ Procedure de sortie }

Procedure ExitXSer; Far; Assembler;
Asm
    MOV  AX, [WORD PTR SaveExit]
    MOV  [WORD PTR ExitProc], AX
    MOV  AX, [WORD PTR SaveExit+2]
    MOV  [WORD PTR ExitProc+2], AX      { ExitProc := SaveExit }

    { Vire les checkup }

    CALL KillCheckUp

    CALL KillCheckUpAT

    CALL CloseAll          { Ferme toutes les communications }
End;



Begin
  { Initialise le tableau des Irq+RegBase }
  Asm
      PUSH DS

      MOV  AX, SEG TabIrq
      MOV  DS, AX
      MOV  SI, OFFSET TabIrq

      MOV  CX, MAXIRQ

      XOR  AX, AX
  @ICI:
      MOV  [SI+WORD PTR IrqDef.IntVec], AX
      MOV  [SI+WORD PTR IrqDef.IntVec+2], AX
      MOV  [SI+IrqDef.IrqMsk], AL
      MOV  [SI+WORD PTR IrqDef.FirstCom], AX
      MOV  [SI+WORD PTR IrqDef.FirstCom+2], AX
      MOV  [SI+WORD PTR IrqDef.LastCom], AX
      MOV  [SI+WORD PTR IrqDef.LastCom+2], AX

      ADD  SI, 16
      LOOP @ICI

      POP  DS

      MOV  [CheckUpImbr], 1
      MOV  [XSerIsClose], FALSE
      MOV  AL, NOCHECKUP
      MOV  [SpeedXT], AL
      MOV  [SpeedAT], AL

      MOV  AX, [WORD PTR ExitProc]
      MOV  [WORD PTR SaveExit], AX
      MOV  AX, [WORD PTR ExitProc+2]
      MOV  [WORD PTR SaveExit+2], AX    { SaveExit := ExitProc (sauve) }

      MOV  AX, SEG ExitXSer
      MOV  [WORD PTR ExitProc+2], AX
      MOV  AX, OFFSET ExitXSer
      MOV  [WORD PTR ExitProc], AX      { ExitProc := @ExitXSer }
  End;
End.
